'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _utils = require('./utils');

var utils = _interopRequireWildcard(_utils);

var _postProcessor = require('./postProcessor');

var _postProcessor2 = _interopRequireDefault(_postProcessor);

var _compatibility110X = require('./compatibility/1.10.x');

var compat = _interopRequireWildcard(_compatibility110X);

var Translator = (function () {
  function Translator(services) {
    var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

    _classCallCheck(this, Translator);

    if (services.resourceStore) this.resourceStore = services.resourceStore;
    if (services.languageUtils) this.languageUtils = services.languageUtils;
    if (services.pluralResolver) this.pluralResolver = services.pluralResolver;
    if (services.interpolator) this.interpolator = services.interpolator;
    if (services.backendConnector) this.backendConnector = services.backendConnector;
    this.options = options;
  }

  _createClass(Translator, [{
    key: 'changeLanguage',
    value: function changeLanguage(lng) {
      if (lng) this.language = lng;
    }
  }, {
    key: 'exists',
    value: function exists(key) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? { interpolation: {} } : arguments[1];

      return this.resolve(key, options) !== undefined;
    }
  }, {
    key: 'translate',
    value: function translate(keys) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (options && typeof options !== 'object') {
        options = this.options.overloadTranslationOptionHandler(arguments);
      } else {
        options = options || {};
      }

      if (this.options.compatibilityAPI === '1.10.x') {
        options = compat.convertTOptions(options);
      }

      // non valid keys handling
      if (keys === undefined || keys === null || keys === '') return '';
      if (typeof keys === 'number') keys = String(keys);
      if (typeof keys === 'string') keys = [keys];
      var key = keys[keys.length - 1];

      // return key on CIMode
      if (this.language === 'CIMode') return key;

      // get namespace(s)
      var namespaces = options.ns || this.options.defaultNS;
      if (key.indexOf(':' /*o.nsseparator*/) > -1) {
        var parts = key.split(':');
        namespaces = parts[0];
        key = parts[1];
      }
      if (typeof namespaces === 'string') namespaces = [namespaces];
      var namespace = namespaces[namespaces.length - 1];

      // resolve from store
      var res = this.resolve(keys, options);

      var resType = Object.prototype.toString.apply(res);
      var noObject = ['[object Number]', '[object Function]', '[object RegExp]'];

      // object
      if (res && typeof res !== 'string' && noObject.indexOf(resType) < 0) {
        // TODO: log or emit this
        if (!options.returnObjects && !this.options.returnObjects) {
          return this.options.returnedObjectHandler ? this.options.returnedObjectHandler(key, res, options) : 'key \'' + key + ' (' + this.language + ')\' returned an object instead of string.';
        }

        var copy = resType === '[object Array]' ? [] : {}; // apply child translation on a copy

        for (var m in res) {
          // TODO: seperator support not on dot
          copy[m] = this.translate(key + '.' + m, options);
        }
        res = copy;
      }
      // string, empty or null
      else {
          var usedDefault = false,
              usedKey = false;

          // fallback value
          if (!this.isValidLookup(res) && options.defaultValue) {
            usedDefault = true;
            res = options.defaultValue;
          }
          if (!this.isValidLookup(res)) {
            usedKey = true;
            res = key;
          }

          // save missing
          if (this.options.saveMissing) {
            var lngs = [];
            if (this.options.saveMissingTo === 'fallback' && this.options.fallbackLng[0]) {
              for (var i = 0; i < this.options.fallbackLng.length; i++) {
                lngs.push(this.options.fallbackLng[i]);
              }
            } else if (this.options.saveMissingTo === 'all') {
              lngs = this.languageUtils.toResolveHierarchy(options.lng || this.language);
            } else {
              //(this.options.saveMissingTo === 'current' || (this.options.saveMissingTo === 'fallback' && this.options.fallbackLng[0] === false) ) {
              lngs.push(options.lng || this.language);
            }

            if (this.options.missingKeyHandler) {
              this.options.missingKeyHandler(lngs, namespace, key, res);
            } else if (this.backendConnector && this.backendConnector.saveMissing) {
              this.backendConnector.saveMissing(lngs, namespace, key, res);
            }
          }

          // extend
          res = this.extendTranslation(res, key, options);

          // append namespace if still key
          if (this.options.appendNamespaceToMissingKey && res === key) res = namespace + ':' + key;

          // parseMissingKeyHandler
          if (usedKey && this.options.parseMissingKeyHandler) res = this.options.parseMissingKeyHandler(res);
        }

      // return
      return res;
    }
  }, {
    key: 'extendTranslation',
    value: function extendTranslation(res, key, options) {
      var _this = this;

      if (options.interpolation) this.interpolator.init(options);
      // nesting
      res = this.interpolator.nest(res, function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return _this.translate.apply(_this, args);
      }, options);

      // interpolate
      var data = options.replace && typeof options.replace !== 'string' ? options.replace : options;
      if (this.options.interpolation.defaultVariables) data = utils.defaults(data, this.options.interpolation.defaultVariables);
      res = this.interpolator.interpolate(res, data);
      if (options.interpolation) this.interpolator.reset();

      // post process
      var postProcess = options.postProcess || this.options.postProcess;
      var postProcessorNames = typeof postProcess === 'string' ? [postProcess] : postProcess;

      if (res !== undefined && postProcessorNames && postProcessorNames.length) res = _postProcessor2['default'].handle(postProcessorNames, res, key, options);

      return res;
    }
  }, {
    key: 'resolve',
    value: function resolve(keys) {
      var _this2 = this;

      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var found = undefined;

      if (typeof keys === 'string') keys = [keys];

      // forEach possible key
      keys.forEach(function (key) {
        if (_this2.isValidLookup(found)) return;

        var namespaces = options.ns || _this2.options.defaultNS;
        if (key.indexOf(':' /*o.nsseparator*/) > -1) {
          var parts = key.split(':');
          namespaces = parts[0];
          key = parts[1];
        }
        if (typeof namespaces === 'string') namespaces = [namespaces];
        if (_this2.options.fallbackNS) namespaces = namespaces.concat(_this2.options.fallbackNS);

        var needsPluralHandling = options.count !== undefined && typeof options.count !== 'string';
        var needsContextHandling = options.context !== undefined && typeof options.context === 'string' && options.context !== '';

        var codes = _this2.languageUtils.toResolveHierarchy(options.lng || _this2.language);
        if (options.lngs) codes = options.lngs;

        namespaces.forEach(function (ns) {
          if (_this2.isValidLookup(found)) return;

          codes.forEach(function (code) {
            if (_this2.isValidLookup(found)) return;

            var finalKey = key;
            var finalKeys = [finalKey];

            // get key for context if needed
            if (needsContextHandling) finalKeys.push(finalKey += '_' + options.context);

            // get key for plural if needed
            if (needsPluralHandling) finalKeys.push(finalKey += _this2.pluralResolver.getSuffix(code, options.count));

            // iterate over finalKeys starting with most specific pluralkey (-> contextkey only) -> singularkey only
            var possibleKey = undefined;
            while (possibleKey = finalKeys.pop()) {
              if (_this2.isValidLookup(found)) continue;
              found = _this2.getResource(code, ns, possibleKey, options);
            }
          });
        });
      });

      return found;
    }
  }, {
    key: 'isValidLookup',
    value: function isValidLookup(res) {
      return res !== undefined && !(!this.options.returnNull && res === null) && !(!this.options.returnEmptyString && res === '');
    }
  }, {
    key: 'getResource',
    value: function getResource(code, ns, key) {
      var options = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

      return this.resourceStore.getResource(code, ns, key);
    }
  }]);

  return Translator;
})();

exports['default'] = Translator;
module.exports = exports['default'];